using System;
using System.Collections;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Reflection;
using TMPro;
using Unity;
using UnityEngine;
using UnityEngine.Events;
using UnityEngine.SceneManagement;
using UnityEngine.UI;

namespace Mod
{

    public class PoseManager
    {
        public static string path;
        public static List<string> hiddenPoses = new List<string>();
        public static void OnLoad()
        {
            if (!Directory.Exists(@"Mods\\SavedPoses"))
            {
                DialogBox dialogBox = (DialogBox)null;
                DialogBox discordInvite = (DialogBox)null;
                dialogBox = DialogBoxManager.Dialog("Hi, you have installed the Pose Manager mod which adds a lot of features to work with poses.\nA lot of time was spent on the mod and I'm glad you're reading this.\nI suggest you watch the tutorial on how to work with this mod before you go to make incredible animations and poses.\nPlease note - this message will not be shown again.", new DialogButton[]
                {
                    new DialogButton("Yes, I want", true, () =>
                    {
                        OpenLink("https://www.youtube.com/watch?v=aur4ElYtwzM");
                        ShowDialog(discordInvite);
                    }),
                    new DialogButton("No thanks, I'll take care of it myself.", true, () =>
                    {
                        dialogBox.Close();
                        ShowDialog(discordInvite);
                    })
                });
                discordInvite = DialogBoxManager.Dialog("One more thing. A lot of effort and time has been put into this mod, and I would be glad if I didn't have your feedback.", new DialogButton[]
                {
                    new DialogButton("Join the community on Discord", false, () =>
                    {
                        OpenLink("https://discord.gg/tUvF8jmbWm");
                    }),
                    new DialogButton("Support Team 01 (Patreon analog)", false, () =>
                    {
                        OpenLink("https://boosty.to/dyadyushkadyadya");
                    }),
                    new DialogButton("Close forever", true, () => DialogBoxManager.Notification("Thanks for installing the mod. \nYours 01 Studio\nThese messages will no longer appear (unless you delete the folders with poses and animations)"))
                });
                HideDialog(discordInvite);
            }

            CheckExsistDirectory();
        }
        public static void CheckExsistDirectory()
        {
            if (!Directory.Exists(@"Mods\\SavedPoses"))
            {
                Directory.CreateDirectory(@"Mods\\SavedPoses");
            }
            if (!Directory.Exists(@"Mods\\SavedPoses\\SavedAnimations"))
            {
                Directory.CreateDirectory(@"Mods\\SavedPoses\\SavedAnimations");
            }
        }
        public static void Main()
        {
            ModAPI.OnItemSpawned += ModAPI_OnItemSpawned;
        }
        private static void AddDeletePoseButton(PersonBehaviour personBehaviour)
        {
            var contextButton = new ContextMenuButton(() => IsSelectedOnlyHim(personBehaviour.Limbs[0].PhysicalBehaviour), "deletePose", "Delete pose", "Delete pose", () =>
            {
                DialogBox dialog = (DialogBox)null;
                dialog = DialogBoxManager.TextEntry("Name pose", "Enter here name your pose for delete", new DialogButton[] { new DialogButton("Delete", true, () =>
                {
                    if (GetAllPosesName().Contains(dialog.EnteredText))
                    {
                        try
                        {
                            ModAPI.DeleteJSON("SavedPoses\\" + dialog.EnteredText + ".json");
                            ModAPI.Notify("Pose " + dialog.EnteredText + " successfully deleted!");
                            ModAPI.Notify("The pose will no longer appear on creatures.");
                        }
                        catch{}
                    }
                    else
                    {
                        ModAPI.Notify("<color=\"red\">There is no pose by that name.");
                    }
                }), new DialogButton("Cancel", true, () => { dialog.Close(); })});
            });
            personBehaviour.Limbs[0].PhysicalBehaviour.ContextMenuOptions.Buttons.Add(contextButton);
        }
        private static void AddDeleteAnimationButton(PersonBehaviour personBehaviour)
        {
            var contextButton = new ContextMenuButton(() => IsSelectedOnlyHim(personBehaviour.Limbs[0].PhysicalBehaviour), "deleteAnimation", "Delete animation", "Delete animation", () =>
            {
                bool deletePoses = true;
                DialogBox dialog = (DialogBox)null;
                DialogBox deleteOrSavePoses = (DialogBox)null;
                deleteOrSavePoses = DialogBoxManager.Dialog("Remove or return the poses that were used in the animation\nSaved poses will become available again for use, as well as for creating new animations.", new DialogButton[]
                {
                    new DialogButton("Delete", true, () =>
                    {
                        deletePoses = true;
                        ShowDialog(dialog);
                    }),
                    new DialogButton("Save", true, () =>
                    {
                        deletePoses = false;
                        ShowDialog(dialog);
                    }),
                    new DialogButton("Cancel", true, () =>
                    {
                        dialog.Close();
                        deleteOrSavePoses.Close();
                    })
                });


                dialog = DialogBoxManager.TextEntry("Name animation", "Enter here name your animation for delete", new DialogButton[] { new DialogButton("Delete", true, () =>
                {
                    if (GetAllAnimationsName().Contains(dialog.EnteredText))
                    {
                        try
                        {
                            var namePoses = ModAPI.DeserialiseJSON<RagdollAnimation>("SavedPoses\\SavedAnimations\\" + dialog.EnteredText + ".json").posesName;
                            if (deletePoses)
                            {
                                foreach(var name in namePoses)
                                {
                                    ModAPI.DeleteJSON("SavedPoses\\" + name + ".json");
                                }
                            }
                            else
                            {
                                foreach(var name in namePoses)
                                {
                                    var ragdollDesirialize = ModAPI.DeserialiseJSON<RagdollSerialize>("SavedPoses\\" + name + ".json");
                                    ragdollDesirialize.Hidden = false;
                                    hiddenPoses.Remove(name);
                                    ModAPI.SerialiseJSON(ragdollDesirialize, "SavedPoses\\" + name + ".json");
                                    foreach(var personLocal in GameObject.FindObjectsOfType<PersonBehaviour>())
                                    {
                                        InitializePoseFromJson(personLocal, name);
                                    }
                                }
                            }
                            ModAPI.DeleteJSON("SavedPoses\\SavedAnimations\\" + dialog.EnteredText + ".json");
                            ModAPI.Notify("Animation " + dialog.EnteredText + " successfully deleted!");
                            ModAPI.Notify("The animation will no longer appear on creatures.");
                        }
                        catch{}
                    }
                    else
                    {
                        ModAPI.Notify("<color=\"red\">There is no animation by that name.");
                    }
                }), new DialogButton("Cancel", true, () => { dialog.Close(); })});
                HideDialog(dialog);
            });
            personBehaviour.Limbs[0].PhysicalBehaviour.ContextMenuOptions.Buttons.Add(contextButton);
        }
        public static void OpenLink(string url)
        {
            Type type = Type.GetType("UnityEngine.Application, UnityEngine.CoreModule");
            type.GetMethod("OpenURL", BindingFlags.Public | BindingFlags.Instance | BindingFlags.Static).Invoke(null, new object[] { url });
        }
        public static void ShowDialog(DialogBox dialogBox)
        {
            dialogBox.gameObject.SetActive(true);
        }
        public static void HideDialog(DialogBox dialogBox)
        {
            dialogBox.gameObject.SetActive(false);
        }
        private static void AddCreateAnimationButton(PersonBehaviour personBehaviour)
        {
            var contextButton = new ContextMenuButton(() => IsSelectedOnlyHim(personBehaviour.Limbs[0].PhysicalBehaviour), "createAnimation", "Create new animation", "Create new animation", () =>
            {
                CheckExsistDirectory();
                if (!Global.main.Paused)
                {
                    Global.main.TogglePaused();
                }
                List<string> namePoses = new List<string>();
                float delay = 1;
                bool looping = false;
                DialogBox dialog1 = (DialogBox)null;
                DialogBox dialog2 = (DialogBox)null;
                DialogBox dialog3 = (DialogBox)null;
                DialogBox dialogFinal = (DialogBox)null;
                RagdollSerialize.TypeCreature? typeCreature = null;

                dialog1 = DialogBoxManager.TextEntry("Enter the names of all the poses here, one by one, in the order in which they are to be played and add them one by one. \nThe poses must be created with this mod, vanilla poses or poses from other mods will not work. \nOnce you're done, click the next button.", "Name pose", new DialogButton[]
                {
                    new DialogButton("Add pose", false, () =>
                    {
                        if (GetAllPosesName().Contains(dialog1.EnteredText))
                        {
                            if(typeCreature == null)
                            {
                                typeCreature = GetRagdollSerialize(dialog1.EnteredText).typeCreature;
                                ModAPI.Notify("Type of creature animation: " + (typeCreature == RagdollSerialize.TypeCreature.Humanoid ? "Humanoid" : "Other creature"));
                            }
                            if(typeCreature == GetRagdollSerialize(dialog1.EnteredText).typeCreature)
                            {
                                if(typeCreature == RagdollSerialize.TypeCreature.CustomCreature)
                                {
                                    if(personBehaviour.name == GetRagdollSerialize(dialog1.EnteredText).NameCreature)
                                    {
                                        var ragdollDesirialize = GetRagdollSerialize(dialog1.EnteredText);
                                        if (ragdollDesirialize.Hidden || namePoses.Contains(ragdollDesirialize.Name))
                                        {
                                            ModAPI.Notify("<color=\"red\">This pose is already used in another animation");
                                        }
                                        else
                                        {
                                            namePoses.Add(dialog1.EnteredText);
                                            ModAPI.Notify("Pose " + dialog1.EnteredText + " successfully added to the animation");
                                        }
                                    }
                                    else
                                    {
                                        ModAPI.Notify("<color=\"red\">The creature name does not equal the creature name when creating a pose");
                                    }
                                }
                                else
                                {
                                    var ragdollDesirialize = GetRagdollSerialize(dialog1.EnteredText);
                                    if (ragdollDesirialize.Hidden || namePoses.Contains(ragdollDesirialize.Name))
                                    {
                                        ModAPI.Notify("<color=\"red\">This pose is already used in another animation");
                                    }
                                    else
                                    {
                                        namePoses.Add(dialog1.EnteredText);
                                        ModAPI.Notify("Pose " + dialog1.EnteredText + " successfully added to the animation");
                                    }
                                }
                            }
                            else
                            {
                                ModAPI.Notify("<color=\"red\">The pose you are trying to add does not match the type of creature animation.");
                            }
                        }
                        else
                        {
                            ModAPI.Notify("<color=\"red\">Pose " + dialog1.EnteredText + " does not exist");
                        }
                    }),
                    new DialogButton("Next", true, () =>
                    {
                        if(namePoses.Count < 2)
                        {
                            ModAPI.Notify("<color=\"red\">There must be at least 2 poses in the animation!");
                        }
                        else
                        {
                            ShowDialog(dialog2);
                        }
                    }),
                    new DialogButton("Cancel", true, () => dialog1.Close())
                });
                dialog2 = DialogBoxManager.TextEntry("Set a delay between pose changes. Standard = 1 sec.", "Enter delay (seconds)", new DialogButton[]
                {
                    new DialogButton("Skip", true, () => { delay = 1; ShowDialog(dialog3); }), new DialogButton("Set delay", true, () =>
                    {
                        try
                        {
                            delay = (float)Convert.ToDouble(dialog2.EnteredText);
                        }
                        catch
                        {
                            ModAPI.Notify("You entered an invalid value");
                            delay = 1;
                        }
                        ShowDialog(dialog3);
                    })
                });
                HideDialog(dialog2);
                dialog3 = DialogBoxManager.Dialog("Should the animation loop?", new DialogButton[]
                {
                    new DialogButton("Yes", true, () =>
                    {
                        looping = true;
                        ShowDialog(dialogFinal);
                    }),
                    new DialogButton("No", true, () =>
                    {
                        looping = false;
                        ShowDialog(dialogFinal);
                    })
                });
                HideDialog(dialog3);
                dialogFinal = DialogBoxManager.TextEntry("Name animation", "Enter here name your animation", new DialogButton[] { new DialogButton("Create", false, () =>
                {
                    if(dialogFinal.EnteredText != "")
                    {
                        if (!GetAllAnimationsName().Contains(dialogFinal.EnteredText))
                        {
                            CheckExsistDirectory();
                            var ragdollAnimation = new RagdollAnimation(namePoses, delay, dialogFinal.EnteredText, looping);
                            SaveAnimationToJson(ragdollAnimation);
                            Debug.Log(personBehaviour.Limbs[0].gameObject.GetComponent<RagdollAnimationController>());
                            ModAPI.Notify("<color=\"green\"><size=150%>Create a new creature and your animation is ready to use!");
                            if (Global.main.Paused)
                            {
                                Global.main.TogglePaused();
                            }
                            dialogFinal.Close();
                        }
                        else
                        {
                            ModAPI.Notify("<color=\"red\">A animation with this name already exists, delete the pose and try again.");
                        }
                    }
                    else
                    {
                        ModAPI.Notify("<color=\"red\">The name of the animation must not be empty.");
                    }

                }), new DialogButton("Cancel", true, () =>
                {
                    dialogFinal.Close();
                    if (Global.main.Paused)
                    {
                        Global.main.TogglePaused();
                    }
                })});
                HideDialog(dialogFinal);

            });
            personBehaviour.Limbs[0].PhysicalBehaviour.ContextMenuOptions.Buttons.Add(contextButton);
        }
        private static void AddCreatePoseButton(PersonBehaviour personBehaviour)
        {
            var contextButton = new ContextMenuButton(() => IsSelectedOnlyHim(personBehaviour.Limbs[0].PhysicalBehaviour), "createPose", "Create new pose", "Create new pose", () =>
            {
                CheckExsistDirectory();
                if (!Global.main.Paused)
                {
                    Global.main.TogglePaused();
                }
                bool shouldStandUpright = true;
                float force = 1;
                float forceUpright = 1;
                RagdollSerialize.TypeCreature? typeCreature = null;
                string nameCreature = "Human";
                DialogBox dialog1 = (DialogBox)null;
                DialogBox dialog2 = (DialogBox)null;
                DialogBox dialog3 = (DialogBox)null;
                DialogBox dialog4 = (DialogBox)null;
                DialogBox dialogFinal = (DialogBox)null;
                dialog1 = DialogBoxManager.Dialog("Does the creature have to keep its balance and stay on its feet?", new DialogButton[]
                {
                    new DialogButton("Yes", true, () => {  shouldStandUpright = true; ShowDialog(dialog2); }), new DialogButton("No", true, () => { shouldStandUpright = false; ShowDialog(dialog2); })
                });
                dialog2 = DialogBoxManager.TextEntry("Force multiplier. Standard = 1.", "Enter multiploer float", new DialogButton[]
                {
                    new DialogButton("Skip", true, () => { force = 1; ShowDialog(dialog3); }), new DialogButton("Set force", true, () =>
                    {
                        try
                        {
                            force = (float)Convert.ToDouble(dialog2.EnteredText);
                        }
                        catch
                        {
                            ModAPI.Notify("You entered an invalid value");
                            force = 1;
                        }
                        ShowDialog(dialog3);
                    })
                });
                HideDialog(dialog2);

                dialog3 = DialogBoxManager.TextEntry("Force of balance multiplier. Standard = 1.", "Enter multiplier float", new DialogButton[]
                {
                    new DialogButton("Skip", true, () => { forceUpright = 1; ShowDialog(dialog4); }), new DialogButton("Set force", true, () =>
                    {
                        try
                        {
                            forceUpright = (float)Convert.ToDouble(dialog3.EnteredText);
                        }
                        catch
                        {
                            ModAPI.Notify("You entered an invalid value");
                            forceUpright = 1;
                        }
                        ShowDialog(dialog4);
                    })
                });
                HideDialog(dialog3);
                dialog4 = DialogBoxManager.Dialog("Select the type of creature.\nThis will determine the availability of the pose.\nAll humanoids will have access to all animations for humanoids.\nOther creatures should have unique poses.", new DialogButton[]
                {
                    new DialogButton("Humanoid", false, () =>
                    {
                        if(personBehaviour.Limbs.Length != 14)
                        {
                            ModAPI.Notify("<color=\"red\">The creature has more or fewer than 14 limbs, it is not a humanoid");
                        }
                        else
                        {
                            typeCreature = RagdollSerialize.TypeCreature.Humanoid;
                            ShowDialog(dialogFinal);
                            dialog4.Close();
                        }
                    }),
                    new DialogButton("Other creature", true, () =>
                    {
                        nameCreature = personBehaviour.name;
                        typeCreature = RagdollSerialize.TypeCreature.CustomCreature;
                        ShowDialog(dialogFinal);
                    })
                });
                HideDialog(dialog4);
                dialogFinal = DialogBoxManager.TextEntry("Name pose", "Enter here name your pose", new DialogButton[] { new DialogButton("Create", false, () =>
                {
                    if(dialogFinal.EnteredText != "")
                    {
                        if (!GetAllPosesName().Contains(dialogFinal.EnteredText))
                        {
                            personBehaviour.CreatePose();
                            var pose = personBehaviour.Poses.Last();
                            pose.Name = dialogFinal.EnteredText;
                            pose.ShouldStandUpright = shouldStandUpright;
                            pose.ForceMultiplier = force;
                            pose.UprightForceMultiplier = forceUpright;
                            Debug.Log(typeCreature);
                            CheckExsistDirectory();
                            SavePoseToJson(pose, typeCreature, nameCreature);
                            InitializePoseFromJson(personBehaviour, pose.Name);
                            if (Global.main.Paused)
                            {
                                Global.main.TogglePaused();
                            }
                            dialogFinal.Close();
                        }
                        else
                        {
                            ModAPI.Notify("<color=\"red\">A pose with this name already exists, delete the pose and try again.");
                        }
                    }
                    else
                    {
                        ModAPI.Notify("<color=\"red\">The name of the pose must not be empty.");
                    }

                }), new DialogButton("Cancel", true, () =>
                {
                    dialogFinal.Close();
                    if (Global.main.Paused)
                    {
                        Global.main.TogglePaused();
                    }
                })});
                HideDialog(dialogFinal);
            });
            personBehaviour.Limbs[0].PhysicalBehaviour.ContextMenuOptions.Buttons.Add(contextButton);
        }
        private static void ModAPI_OnItemSpawned(object sender, UserSpawnEventArgs e)
        {
            if (e.Instance.gameObject.TryGetComponent<PersonBehaviour>(out PersonBehaviour personBehaviour))
            {
                // todo   
                AddCreatePoseButton(personBehaviour);
                AddDeletePoseButton(personBehaviour);
                AddCreateAnimationButton(personBehaviour);
                AddDeleteAnimationButton(personBehaviour);
                AddAllPosesFromDirectory(personBehaviour);

                personBehaviour.Limbs[0].gameObject.AddComponent<RagdollAnimationController>();
            }
        }
        public static bool IsSelectedOnlyHim(PhysicalBehaviour physicalBehaviour)
        {
            if (SelectionController.Main.SelectedObjects.Count == 1)
            {
                if (SelectionController.Main.SelectedObjects[0] == physicalBehaviour)
                {
                    return true;
                }
                else
                {
                    return false;
                }
            }
            else
            {
                return false;
            }
        }
        private static void AddAllPosesFromDirectory(PersonBehaviour personBehaviour)
        {
            string[] namesPoses = Directory.GetFiles(@"Mods\\SavedPoses");

            foreach (var name in namesPoses)
            {
                var nameWithoutPath = System.IO.Path.GetFileNameWithoutExtension(name);
                var ragdollDeserialize = GetRagdollSerialize(nameWithoutPath);
                if (ragdollDeserialize.typeCreature == RagdollSerialize.TypeCreature.Humanoid && personBehaviour.Limbs.Length == 14)
                {
                    InitializePoseFromJson(personBehaviour, nameWithoutPath);
                }
                else if (ragdollDeserialize.typeCreature == RagdollSerialize.TypeCreature.CustomCreature && personBehaviour.name == ragdollDeserialize.NameCreature)
                {
                    InitializePoseFromJson(personBehaviour, nameWithoutPath);
                }
            }
        }
        private static RagdollAnimation GetRagdollAnimation(string nameAnimation)
        {
            var animationDesirialize = ModAPI.DeserialiseJSON<RagdollAnimation>("SavedPoses\\SavedAnimations\\" + nameAnimation + ".json");
            return animationDesirialize;
        }
        public static void AddAllAnimationsFromDirectory(PersonBehaviour personBehaviour, RagdollAnimationController animationController)
        {
            string[] namesAnimations = Directory.GetFiles(@"Mods\\SavedPoses\\SavedAnimations");
            foreach (var name in namesAnimations)
            {
                var nameWithoutPath = Path.GetFileNameWithoutExtension(name);
                var firstRagdollInAnimation = GetRagdollSerialize(GetRagdollAnimation(nameWithoutPath).posesName[0]);
                if (firstRagdollInAnimation.typeCreature == RagdollSerialize.TypeCreature.Humanoid && personBehaviour.Limbs.Length == 14)
                {
                    InitializeAnimationFromJson(personBehaviour, nameWithoutPath, animationController);
                }
                else if (firstRagdollInAnimation.typeCreature == RagdollSerialize.TypeCreature.CustomCreature && personBehaviour.name == firstRagdollInAnimation.NameCreature)
                {
                    InitializeAnimationFromJson(personBehaviour, nameWithoutPath, animationController);
                }
            }
        }
        public static List<string> GetAllPosesName()
        {
            var list = new List<string>();
            string[] namesPoses = Directory.GetFiles(@"Mods\\SavedPoses");
            foreach (var name in namesPoses)
            {
                var nameWithoutPath = System.IO.Path.GetFileNameWithoutExtension(name);
                list.Add(nameWithoutPath);
            }
            return list;
        }
        public static List<string> GetAllAnimationsName()
        {
            var list = new List<string>();
            string[] namesPoses = Directory.GetFiles(@"Mods\\SavedPoses\\SavedAnimations");
            foreach (var name in namesPoses)
            {
                var nameWithoutPath = System.IO.Path.GetFileNameWithoutExtension(name);
                list.Add(nameWithoutPath);
            }
            return list;
        }
        public static void SavePoseToJson(RagdollPose ragdollPose, RagdollSerialize.TypeCreature? typeCreature = RagdollSerialize.TypeCreature.Humanoid, string nameCreation = "Human")
        {
            var ragdollPoseSerialize = new RagdollSerialize(ragdollPose.Name, ragdollPose.Angles, ragdollPose.AnimationSpeedMultiplier, ragdollPose.DragInfluence, ragdollPose.ForceMultiplier, ragdollPose.ShouldStandUpright, ragdollPose.ShouldStumble, ragdollPose.Rigidity, ragdollPose.UprightForceMultiplier, false, typeCreature, nameCreation);
            ModAPI.SerialiseJSON(ragdollPoseSerialize, "SavedPoses\\" + ragdollPoseSerialize.Name + ".json");
            ModAPI.Notify("Pose " + ragdollPose.Name + " successfully saved!");
        }
        public static void SaveAnimationToJson(RagdollAnimation ragdollAnimation)
        {
            foreach (var pose in ragdollAnimation.posesName)
            {
                var ragdollDesirialize = ModAPI.DeserialiseJSON<RagdollSerialize>("SavedPoses\\" + pose + ".json");
                if (!hiddenPoses.Contains(pose))
                {
                    hiddenPoses.Add(pose);
                }
                ragdollDesirialize.Hidden = true;
                ModAPI.SerialiseJSON(ragdollDesirialize, "SavedPoses\\" + pose + ".json");
            }
            ModAPI.SerialiseJSON(ragdollAnimation, "SavedPoses\\SavedAnimations\\" + ragdollAnimation.name + ".json");
            ModAPI.Notify("Animation " + ragdollAnimation.name + " successfully saved!");
        }

        public static RagdollPose InitializePoseFromJson(PersonBehaviour personBehaviour, string namePose)
        {
            var ragdollPose = new RagdollPose();
            var ragdollDesirialize = ModAPI.DeserialiseJSON<RagdollSerialize>("SavedPoses\\" + namePose + ".json");
            // next hell >:#

            var desirializeAngles = new List<RagdollPose.LimbPose>();
            foreach (var angle in ragdollDesirialize.Angles)
            {
                var angleDesirialize = new RagdollPose.LimbPose(FindLimb(personBehaviour, angle.Name), angle.Angle);
                desirializeAngles.Add(angleDesirialize);
            }
            //init fields
            ragdollPose.Angles = desirializeAngles;
            ragdollPose.AnimationSpeedMultiplier = ragdollDesirialize.AnimationSpeedMultiplier;
            ragdollPose.DragInfluence = ragdollDesirialize.DragInfluence;
            ragdollPose.ForceMultiplier = ragdollDesirialize.ForceMultiplier;
            ragdollPose.Name = ragdollDesirialize.Name;
            ragdollPose.Rigidity = ragdollDesirialize.Rigdity;
            ragdollPose.ShouldStandUpright = ragdollDesirialize.ShouldStandUpright;
            ragdollPose.ShouldStumble = ragdollDesirialize.ShouldStumble;
            ragdollPose.State = PoseState.Sitting;
            ragdollPose.UprightForceMultiplier = ragdollDesirialize.UprightForceMultyiplier;

            if (ragdollDesirialize.Hidden && !hiddenPoses.Contains(ragdollDesirialize.Name))
            {
                hiddenPoses.Add(ragdollDesirialize.Name);
            }
            personBehaviour.Poses.Add(ragdollPose);
            ragdollPose.ConstructDictionary();
            var poseID = personBehaviour.Poses.IndexOf(ragdollPose);
            AddPoseButton(personBehaviour, CreateChangePoseButton(ragdollPose.Name, poseID, personBehaviour));
            return ragdollPose;
        }
        public static void InitializeAnimationFromJson(PersonBehaviour personBehaviour, string nameAnimation, RagdollAnimationController animationController)
        {
            var animationDesirialize = ModAPI.DeserialiseJSON<RagdollAnimation>("SavedPoses\\SavedAnimations\\" + nameAnimation + ".json");
            var contextButton = new ContextMenuButton(() => !IsHiddenAnimation(nameAnimation), nameAnimation + "animation", nameAnimation, nameAnimation, () =>
            {
                Debug.Log(animationDesirialize.name);
                Debug.Log(animationController);
                animationController.PlayAnimation(animationDesirialize);
            });
            AddPoseButton(personBehaviour, contextButton);
        }
        public static bool IsHiddenPose(string namePose)
        {
            if (hiddenPoses.Contains(namePose))
            {
                return true;
            }
            else
            {
                return false;
            }
        }
        public static RagdollSerialize GetRagdollSerialize(string namePose)
        {
            var ragdollDesirialize = ModAPI.DeserialiseJSON<RagdollSerialize>("SavedPoses\\" + namePose + ".json");
            return ragdollDesirialize;
        }
        public static bool IsHiddenAnimation(string nameAnimation)
        {
            if (GetAllAnimationsName().Contains(nameAnimation))
            {
                return false;
            }
            else
            {
                return true;
            }
        }
        public static ContextMenuButton CreateChangePoseButton(string namePose, int poseID, PersonBehaviour personBehaviour)
        {
            var contextButton = new ContextMenuButton(() => !IsHiddenPose(namePose), namePose + "poseDeserialize", namePose, namePose, () =>
            {
                personBehaviour.OverridePoseIndex = poseID;
            });
            return contextButton;
        }
        public static void AddPoseButton(PersonBehaviour personBehaviour, ContextMenuButton contextMenuButton)
        {
            foreach (var limb in personBehaviour.Limbs)
            {
                limb.PhysicalBehaviour.ContextMenuOptions.Buttons.Add(contextMenuButton);
            }
        }
        public static LimbBehaviour FindLimb(PersonBehaviour personBehaviour, string nameLimb)
        {
            foreach (var limb in personBehaviour.Limbs)
            {
                if (limb.name == nameLimb)
                {
                    return limb;
                }
            }
            return null;
        }
    }
    public class RagdollAnimationController : MonoBehaviour
    {
        public PersonBehaviour personBehaviour;
        public Dictionary<string, int> posesNameKeys = new Dictionary<string, int>(); // poseName - overrideId
        public RagdollAnimation nowAnimation;
        public Coroutine nowCoroutine;
        public string nowPose;
        public int nowOverrideId;

        private void Start()
        {
            personBehaviour = gameObject.GetComponent<LimbBehaviour>().Person;
            PoseManager.AddAllAnimationsFromDirectory(personBehaviour, this);
        }
        public void PlayAnimation(RagdollAnimation ragdollAnimation)
        {
            bool allPosesContains = true;
            List<string> allPoses = PoseManager.GetAllPosesName();
            foreach (var pose in ragdollAnimation.posesName)
            {
                if (!allPoses.Contains(pose))
                {
                    allPosesContains = false;
                    ModAPI.Notify("<color=\"red\">Animation " + ragdollAnimation.name + " not contain pose " + pose);
                }
            }
            if (allPosesContains)
            {
                if (nowCoroutine != null)
                {
                    StopCoroutine(nowCoroutine);
                }
                posesNameKeys.Clear();
                nowAnimation = ragdollAnimation;
                foreach (var pose in nowAnimation.posesName)
                {
                    posesNameKeys.Add(pose, FindOverrideId(pose));
                }
                nowPose = nowAnimation.posesName[0];
                nowOverrideId = posesNameKeys[nowAnimation.posesName[0]];
                personBehaviour.OverridePoseIndex = nowOverrideId;
                nowCoroutine = StartCoroutine(CyclAnimation());
            }
        }
        public void StopAnimation()
        {
            Debug.Log("Animation stopped");
            StopCoroutine(nowCoroutine);
            nowPose = "";
            posesNameKeys.Clear();
        }
        public int FindOverrideId(string namePose)
        {
            foreach (var pose in personBehaviour.Poses)
            {
                if (pose.Name == namePose)
                {
                    return personBehaviour.Poses.IndexOf(pose);
                }
            }
            return -1;
        }
        private int GetNextOverrideId()
        {
            var pose = GetNextPoseName();
            nowPose = pose;
            return posesNameKeys[pose];
        }
        private string GetNextPoseName()
        {
            var nextIndexInPosesNames = nowAnimation.posesName.IndexOf(nowPose) + 1;
            if (nextIndexInPosesNames < nowAnimation.posesName.Count)
            {
                return nowAnimation.posesName[nextIndexInPosesNames];
            }
            else
            {
                if (nowAnimation.looping)
                {
                    return nowAnimation.posesName[0];
                }
                else
                {
                    StopAnimation();
                    return "";
                }

            }
        }
        private IEnumerator CyclAnimation()
        {
            yield return new WaitForSeconds(nowAnimation.delay);
            if (!nowAnimation.posesName.Contains(personBehaviour.Poses[personBehaviour.OverridePoseIndex].Name))
            {
                StopAnimation();
            }
            else
            {
                var overrideID = GetNextOverrideId();
                nowOverrideId = overrideID;
                personBehaviour.OverridePoseIndex = overrideID;
                nowCoroutine = StartCoroutine(CyclAnimation());
            }
        }
    }
    public struct RagdollAnimation
    {
        public List<string> posesName;
        public float delay;
        public string name;
        public bool looping;
        public RagdollAnimation(List<string> posesName, float delay, string nameAnimation, bool looping = true)
        {
            this.posesName = posesName;
            this.delay = delay;
            name = nameAnimation;
            this.looping = looping;
        }
    }
    public struct RagdollSerialize
    {
        public string Name;
        public float AnimationSpeedMultiplier;
        public float DragInfluence;
        public bool ShouldStandUpright;
        public bool ShouldStumble;
        public float ForceMultiplier;
        public float Rigdity;
        public float UprightForceMultyiplier;
        public List<LimbPoseSerialize> Angles;
        public bool Hidden;
        public TypeCreature? typeCreature;
        public string NameCreature;
        public enum TypeCreature
        {
            Humanoid,
            CustomCreature
        }

        public RagdollSerialize(string namePose, List<RagdollPose.LimbPose> angles, float speedAnimation = 1, float dragInfluence = 1, float forceMultiplier = 1, bool shouldStandUpright = true, bool shouldStumble = false, float rigdity = 5, float uprightForceMultiploer = 1, bool hidden = false, TypeCreature? type = TypeCreature.Humanoid, string nameCreature = "Human")
        {
            this.Name = namePose;
            this.AnimationSpeedMultiplier = speedAnimation;
            this.DragInfluence = dragInfluence;
            this.ShouldStandUpright = shouldStandUpright;
            this.ShouldStumble = shouldStumble;
            this.ForceMultiplier = forceMultiplier;
            this.Rigdity = rigdity;
            this.UprightForceMultyiplier = uprightForceMultiploer;
            this.Hidden = hidden;
            this.typeCreature = type;
            this.NameCreature = nameCreature;
            var newAngles = new List<LimbPoseSerialize>();
            foreach (var angle in angles)
            {
                var limbPoseSerialize = new LimbPoseSerialize(angle.Name, angle.Angle);
                newAngles.Add(limbPoseSerialize);
            }
            Angles = newAngles;
        }
        public struct LimbPoseSerialize
        {
            public float Angle;
            public AnimationCurve AnimationCurve;
            public float AnimationDuration;
            public float EndAngle;
            public string Name;
            public float PoseRigidityModifier;
            public float RandomSpeed;
            public float StartAngle;
            public float TimeOffset;
            public float RandomInfluence;
            public LimbPoseSerialize(string nameLimb, float angle)
            {
                this.Angle = angle;
                this.Name = nameLimb;
                this.StartAngle = angle;
                this.EndAngle = angle;
                this.TimeOffset = 0f;
                this.PoseRigidityModifier = 0f;
                this.RandomInfluence = 0f;
                this.RandomSpeed = 1f;
                this.AnimationDuration = 1f;
                this.AnimationCurve = AnimationCurve.EaseInOut(0f, 0f, 1f, 1f);
                this.AnimationCurve.postWrapMode = WrapMode.PingPong;
                this.AnimationCurve.preWrapMode = WrapMode.PingPong;
            }
        }
    }
}